'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactDom = require('react-dom');

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _domLib = require('dom-lib');

var _lodash = require('lodash');

var _Row = require('./Row');

var _Row2 = _interopRequireDefault(_Row);

var _CellGroup = require('./CellGroup');

var _CellGroup2 = _interopRequireDefault(_CellGroup);

var _ClassNameMixin = require('./mixins/ClassNameMixin');

var _ClassNameMixin2 = _interopRequireDefault(_ClassNameMixin);

var _isIE = require('./utils/isIE8');

var _isIE2 = _interopRequireDefault(_isIE);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var ReactChildren = _react2.default.Children;

var Table = _react2.default.createClass({
    displayName: 'Table',

    mixins: [_ClassNameMixin2.default],
    propTypes: {
        width: _react.PropTypes.number,
        data: _react.PropTypes.array.isRequired,
        height: _react.PropTypes.number,
        rowHeight: _react.PropTypes.number,
        headerHeight: _react.PropTypes.number,
        scrollLeft: _react.PropTypes.number,
        scrollTop: _react.PropTypes.number,
        onRowClick: _react.PropTypes.func
    },
    getDefaultProps: function getDefaultProps() {
        return {
            height: 200,
            rowHeight: 36
        };
    },
    getInitialState: function getInitialState() {
        return {
            columnWidth: 0,
            mouseAreaLeft: -1,
            dataKey: 0,
            scrollLeft: 0,
            scrollTop: 0,
            resizeColumnFixed: false
        };
    },
    getFixedCellGroups: function getFixedCellGroups() {
        return (0, _reactDom.findDOMNode)(this.refs.table).querySelectorAll('.' + this.props.classPrefix + '-cell-group.fixed');
    },
    handleBodyScroll: function handleBodyScroll() {
        var _refs = this.refs;
        var tableBody = _refs.tableBody;
        var tableHeader = _refs.tableHeader;

        var tableHeaderDom = (0, _reactDom.findDOMNode)(tableHeader);
        var groups = this.getFixedCellGroups();
        var handelClass = { addClass: _domLib.addClass, removeClass: _domLib.removeClass };

        var left = (0, _domLib.scrollLeft)(tableBody);
        var top = (0, _domLib.scrollTop)(tableBody);

        this.scrollLeft = left;

        Array.from(groups).map(function (group) {
            (0, _domLib.addStyle)(group, { left: left + 'px' });
            var toggle = left > 1 ? 'addClass' : 'removeClass';
            !_isIE2.default && handelClass[toggle](group, 'shadow');
        });

        (0, _domLib.addStyle)(tableHeaderDom, { left: -left + 'px' });

        var toggle = top > 1 ? 'addClass' : 'removeClass';
        !_isIE2.default && handelClass[toggle](tableHeaderDom, 'shadow');
    },
    _onColumnResizeEnd: function _onColumnResizeEnd(columnWidth, cursorDelta, dataKey) {
        this.setState(_defineProperty({
            isColumnResizing: false,
            mouseAreaLeft: -1
        }, dataKey + 'Width', columnWidth));
    },
    _onColumnResize: function _onColumnResize(width, left, event) {
        this.setState({
            isColumnResizing: true
        });
    },
    _onColumnResizeMove: function _onColumnResizeMove(width, left, fixed) {

        this.setState({
            resizeColumnFixed: fixed,
            mouseAreaLeft: width + left
        });
    },
    cloneCell: function cloneCell(Cell, props) {
        return _react2.default.cloneElement(Cell, props, Cell.props.children);
    },
    getCells: function getCells() {
        var _this = this;

        var headerCells = []; // Table header cell
        var bodyCells = []; // Table body cell
        var left = 0; // Cell left margin
        var isFixedColumn = false; // IF there are fixed columns
        var columns = this.props.children;
        var _state = this.state;
        var dataKey = _state.dataKey;
        var columnWidth = _state.columnWidth;


        ReactChildren.map(columns, function (column, index) {

            var columnChildren = column.props.children;
            var _column$props = column.props;
            var width = _column$props.width;
            var fixed = _column$props.fixed;
            var align = _column$props.align;
            var sort = _column$props.sort;
            var resizable = _column$props.resizable;


            if (columnChildren.length !== 2) {
                throw new Error('Component <HeaderCell> and <Cell> is required, column index: ' + index + ' ');
            }

            if (fixed) {
                isFixedColumn = true;
            }

            width = _this.state[columnChildren[1].props.dataKey + 'Width'] || width;

            var cellProps = {
                width: width, fixed: fixed, left: left, align: align, resizable: resizable,
                height: _this.props.rowHeight,
                headerHeight: _this.props.headerHeight,
                firstColumn: index === 0,
                lastColumn: index === columns.length - 1,
                key: index
            };

            var headerCellsProps = {
                dataKey: columnChildren[1].props.dataKey
            };

            if (resizable) {
                headerCellsProps.onColumnResizeEnd = _this._onColumnResizeEnd;
                headerCellsProps.onColumnResize = _this._onColumnResize;
                headerCellsProps.onColumnResizeMove = _this._onColumnResizeMove;
            }

            headerCells.push(_this.cloneCell(columnChildren[0], (0, _lodash.assign)(cellProps, headerCellsProps)));
            bodyCells.push(_this.cloneCell(columnChildren[1], cellProps));

            left += width;
        });

        return {
            headerCells: headerCells,
            bodyCells: bodyCells,
            isFixedColumn: isFixedColumn,
            allColumnsWidth: left
        };
    },
    renderRow: function renderRow(props, cells) {

        //IF there are fixed columns, add a fixed group
        if (this.isFixedColumn) {

            var fixedCells = cells.filter(function (cell) {
                return cell.props.fixed;
            });

            var otherCells = cells.filter(function (cell) {
                return !cell.props.fixed;
            });

            var fixedCellGroupWidth = 0;

            fixedCells.map(function (item) {
                fixedCellGroupWidth += item.props.width;
            });

            return _react2.default.createElement(
                _Row2.default,
                props,
                _react2.default.createElement(
                    _CellGroup2.default,
                    {
                        fixed: true,
                        height: props.isHeaderRow ? props.headerHeight : props.height,
                        width: fixedCellGroupWidth },
                    fixedCells
                ),
                _react2.default.createElement(
                    _CellGroup2.default,
                    null,
                    otherCells
                )
            );
        }

        return _react2.default.createElement(
            _Row2.default,
            props,
            cells
        );
    },
    render: function render() {
        var _props = this.props;
        var children = _props.children;
        var className = _props.className;
        var _props$width = _props.width;
        var width = _props$width === undefined ? 0 : _props$width;
        var height = _props.height;
        var style = _props.style;
        var rowHeight = _props.rowHeight;
        var classPrefix = _props.classPrefix;
        var id = _props.id;

        var _getCells = this.getCells();

        var headerCells = _getCells.headerCells;
        var bodyCells = _getCells.bodyCells;
        var allColumnsWidth = _getCells.allColumnsWidth;
        var isFixedColumn = _getCells.isFixedColumn;

        var rowWidth = allColumnsWidth > width ? allColumnsWidth : width;

        //Check there are fixed columns
        this.isFixedColumn = isFixedColumn;

        var clesses = (0, _classnames2.default)(classPrefix, className, {
            'column-resizing': this.state.isColumnResizing
        });

        var styles = (0, _lodash.assign)({ width: width || 'auto', height: height }, style);

        return _react2.default.createElement(
            'div',
            { className: clesses, style: styles, ref: 'table', id: id },
            this.renderTableHeader(headerCells, rowWidth),
            this.renderTableBody(bodyCells, rowWidth, allColumnsWidth),
            !_isIE2.default && this.renderMouseArea()
        );
    },
    renderTableHeader: function renderTableHeader(headerCells, rowWidth) {
        var _props2 = this.props;
        var rowHeight = _props2.rowHeight;
        var headerHeight = _props2.headerHeight;

        var row = this.renderRow({
            ref: 'tableHeader',
            width: rowWidth,
            height: rowHeight,
            headerHeight: headerHeight,
            isHeaderRow: true,
            top: 0
        }, headerCells);

        return _react2.default.createElement(
            'div',
            {
                className: this.prefix('header-row-wrapper') },
            row
        );
    },
    renderTableBody: function renderTableBody(bodyCells, rowWidth, allColumnsWidth) {
        var _this2 = this;

        var _props3 = this.props;
        var headerHeight = _props3.headerHeight;
        var rowHeight = _props3.rowHeight;
        var height = _props3.height;
        var data = _props3.data;


        var bodyStyles = {
            top: headerHeight || rowHeight,
            height: height - (headerHeight || rowHeight)
        };

        var top = 0; //Row position
        var rows = data.map(function (rowData, index) {

            var cells = bodyCells.map(function (cell, key) {
                return _react2.default.cloneElement(cell, {
                    key: key,
                    rowData: rowData,
                    rowIndex: index
                }, cell.props.children);
            });

            var row = _this2.renderRow({
                key: index,
                rowIndex: index,
                width: rowWidth,
                height: rowHeight,
                rowData: rowData,
                top: top
            }, cells);

            top += rowHeight;
            return row;
        });

        return _react2.default.createElement(
            'div',
            { ref: 'tableBody',
                className: this.prefix('body-row-wrapper'),
                style: bodyStyles },
            rows
        );
    },
    renderMouseArea: function renderMouseArea() {
        var height = this.props.height;

        var scrollLeft = this.scrollLeft || 0;
        var _state2 = this.state;
        var mouseAreaLeft = _state2.mouseAreaLeft;
        var resizeColumnFixed = _state2.resizeColumnFixed;


        var styles = {
            height: height,
            left: resizeColumnFixed ? mouseAreaLeft : mouseAreaLeft - scrollLeft
        };

        return _react2.default.createElement('div', { ref: 'mouseArea', className: this.prefix('mouse-area'), style: styles });
    },
    componentDidMount: function componentDidMount() {
        this._onBodyScrollListener = (0, _domLib.on)(this.refs.tableBody, 'scroll', this.handleBodyScroll);
    },
    componentWillUnmount: function componentWillUnmount() {
        if (this._onBodyScrollListener) {
            this._onBodyScrollListener.off();
        }
    }
});

exports.default = Table;