'use strict';

var _domLib = require('dom-lib');

var TRANSFORM = _domLib.transition.transform;
var BACKFACE_VISIBILITY = _domLib.transition.backfaceVisibility;

var translateDOMPositionXY = function () {
    if (_domLib.BrowserSupportCore.hasCSSTransforms()) {

        var ua = global.window ? global.window.navigator.userAgent : 'UNKNOWN';
        var isSafari = /Safari\//.test(ua) && !/Chrome\//.test(ua);
        // It appears that Safari messes up the composition order
        // of GPU-accelerated layers
        // (see bug https://bugs.webkit.org/show_bug.cgi?id=61824).
        // Use 2D translation instead.
        if (!isSafari && _domLib.BrowserSupportCore.hasCSS3DTransforms()) {
            return function (style, x, y) {
                style[TRANSFORM] = 'translate3d(' + x + 'px,' + y + 'px,0)';
                style[BACKFACE_VISIBILITY] = 'hidden';
            };
        } else {
            return function (style, x, y) {
                style[TRANSFORM] = 'translate(' + x + 'px,' + y + 'px)';
            };
        }
    } else {
        return function (style, x, y) {
            style.left = x + 'px';
            style.top = y + 'px';
        };
    }
}();

module.exports = translateDOMPositionXY;